#pragma once
#include "../Common/Lock.h"
#include "../Common/DateTime.h"
#include <map>
#include <vector>

#ifdef _DEBUG
	#define TSEITPARSER_DEBUG_PRINT
#endif

#define TSEPG_EVENT_TITLE_MAX	120
#define TSEPG_EVENT_TEXT_MAX	200

struct TSEPG_PROGRAM
{
	BYTE	byType;			// f[^^Cv(l̂قADx)
							// 1=p/f ActualɂĎM 2=p/f otherbɂĎM 3=schedule ActualɂĎM 4=Schedule other ɂĎM
	BYTE	byTable;		// e[uԍ[0-7]
	BYTE	byVersion;		// f[^̃o[W

	WORD	wServiceID;		// T[rXID
	WORD	wEventID;		// CxgID
	char	szTitle[TSEPG_EVENT_TITLE_MAX+1];	// ^Cg
	char	szText[TSEPG_EVENT_TEXT_MAX+1];	// 

	DateTime stStart;		// Jn
	DateTime stEnd;			// I
	DWORD	dwDuration;		// ԑg(Srbg=1̏ꍇAI)

	// ȉ͂܂
	struct {
		BYTE byVideoType;		// fR|[lg^Cv
		BYTE byAudioType;		// R|[lg^Cv

	} stComponent;
};

typedef std::map<WORD,struct TSEPG_PROGRAM>		TSEpg_Map;				// Key=CxgID , Val=ԑgƂ̏
typedef std::vector<struct TSEPG_PROGRAM>		TsEpg_Vector;
typedef std::pair<WORD,struct TSEPG_PROGRAM>	TSEpg_MapPair;
typedef struct TSEPG_DATA
{
	BYTE arbyEitSectionVersion[10][4];	// [TableNo][Type]
	TSEpg_Map *pEpgMap; 
} TSEPG_Data;
typedef std::map<WORD,TSEPG_Data*>				TSEpg_ServiceMap;		// Key=T[rXID , Val=T[rXƂ̔ԑg\
typedef std::pair<WORD,TSEPG_Data*>				TSEpg_ServiceMapPair;

#define TSEPG_COMPONENT_VIDEO_D1					0x00	// D1(480i)
#define TSEPG_COMPONENT_VIDEO_D2					0xA0	// D2(480p)
#define TSEPG_COMPONENT_VIDEO_D3					0xB0	// D3(1080i)
//#define TSEPG_COMPONENT_VIDEO_D4					0x00	// D4(720p) l͕s
//#define TSEPG_COMPONENT_VIDEO_D5					0x00	// D5(1080p)l͕s
#define TSEPG_COMPONENT_VIDEO_ASPECT_4_3			0x01	// 4:3
#define TSEPG_COMPONENT_VIDEO_ASPECT_16_9			0x03	// 16:9(pxNg)
#define TSEPG_COMPONENT_VIDEO_ASPECT_16_9_OVER		0x04	// 16:9ȏ

#define TSEPG_COMPONENT_AUDIO_MONO			0x01
#define TSEPG_COMPONENT_AUDIO_DUAL_MONO		0x02
#define TSEPG_COMPONENT_AUDIO_STEREO		0x03
#define TSEPG_COMPONENT_AUDIO_4CH			0x07
#define TSEPG_COMPONENT_AUDIO_5CH			0x08
#define TSEPG_COMPONENT_AUDIO_5_1CH			0x09

//////////////////////////////////////////////////////////////////////
// EPGf[^ǗNX
class CTsEPGMgr
{
public:
	CTsEPGMgr(void);
	~CTsEPGMgr(void);

	void Clear();
	void ClearEpg(WORD wServiceID);

	int GetEpgServiceCount();
	BOOL GetEpgServiceList(WORD **ppList,int *piNum);
	void FreeEpgServiceList(WORD *pList,int iNum);

	TSEPG_Data* GetEpgMap(WORD wServiceID,BOOL bNewFlg=FALSE);
	int GetEpgDataCount(TSEpg_Map *pEpg);
	BOOL GetEpgDataList(TSEpg_Map *pEpg,struct TSEPG_PROGRAM **ppList,int *piNum);
	void FreeEpgDataList(TSEpg_Map *pEpg,struct TSEPG_PROGRAM *pList,int iNum);

	BOOL GetEpgVersion(BYTE byType,WORD wServiceID,BYTE byTableID,BYTE *pbyVersion);
	BOOL SetEpgData(WORD wServiceID,struct TSEPG_PROGRAM *pstNewData);
	struct TSEPG_PROGRAM* FindEpgDataByTime(WORD wServiceID,DateTime &stSearch,BOOL bDeleteMode=FALSE);
	struct TSEPG_PROGRAM* FindEpgDataByEventID(WORD wNetworkID,WORD wStreamID,WORD wServiceID,WORD wEventID);
	void FreeEpgDataMemory(struct TSEPG_PROGRAM *pData,int iCount);
	void DeleteEpgDataByTime(DateTime &stDeleteTime,WORD wServiceID=0xFFFF);
	void DeleteEpgDataByTypeAndTable(WORD wServiceID,BYTE byType,BYTE byTableNo);

	static int TsEitParser(struct TSEPG_PROGRAM **ppstProg,int *piCount,BYTE byTableID,BYTE byVersion,WORD wServiceID,BYTE *pData);
	static void FreeTsEitParserMemory(struct TSEPG_PROGRAM *pstProg,int iCount);
#ifdef TSEITPARSER_DEBUG_PRINT
	static void TsEitParser_DebugPrintProgram(BOOL bActual,struct TS_section_header &stSecHeader,struct TSEPG_PROGRAM &stProg);
#else 
	static inline void TsEitParser_DebugPrintProgram(BOOL bActual,struct TS_section_header &stSecHeader,struct TSEPG_PROGRAM &stProg) { /*Ȃ*/}
#endif

#ifdef _DEBUG
	void DebugDump(WORD wServiceID,char *lpszOutputPath);
#endif

protected:
	LOCK_OBJ	m_EpgLock;
	TSEpg_ServiceMap	m_EpgDB;

	struct TSEPG_PROGRAM* GetEpgData(TSEpg_Map *pEpg,WORD wEventID);
};
